(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2009-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ugen_numbering_hashtbl_print.ml,v 1.10 2017/05/31 16:38:11 fclement Exp $*)

(* Generic numbering hash table printers. *)

open Ugen_numbering_hashtbl;;

let printer_value = Uindex_print.printer_t;;

let pprint_entry pp_key pp_key_value _ctx ppf k kv =
  Format.fprintf ppf "@ @[%a ->@ %a@]"
    (pp_key Og_precedence.Ck_toplevel) k
    (pp_key_value Og_precedence.Ck_toplevel) kv;
;;

let pprint_h pp_key pp_key_value ctx ppf =
  Format.fprintf ppf "%a"
    (fun ppf -> iter (pprint_entry pp_key pp_key_value ctx ppf))
;;

let pprint_entry_inv pp_key pp_key_value _ctx ppf k kv =
  Format.fprintf ppf "@ @[%a ->@ %a@]"
    (pp_key_value Og_precedence.Ck_toplevel) kv
    (pp_key Og_precedence.Ck_toplevel) k;
;;

let pprint_h_inv pp_key pp_key_value ctx ppf =
  Format.fprintf ppf "%a"
    (fun ppf -> iter (pprint_entry_inv pp_key pp_key_value ctx ppf))
;;

let printer_t pp_size pp_key ctx ppf t =
  if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
  then Format.fprintf ppf "%, {@ "
  else Format.fprintf ppf "%,@[<hv 2>{@ ";
  Format.fprintf ppf "%,@[<hv 0>";
  Format.fprintf ppf "%,Ugen_numbering_hashtbl.@,";
  Format.fprintf ppf "%,@[<hv 2>length =";
  Upervasives_print.printer_length
    Og_precedence.Ck_record_field ppf t.length;
  Format.fprintf ppf "%,;@]";
  Format.fprintf ppf "%,@ ";
  Format.fprintf ppf "%,@[<hv 2>size =";
  pp_size Og_precedence.Ck_record_field ppf t.size;
  Format.fprintf ppf "%,;@]";
  Format.fprintf ppf "%,@ ";
  Format.fprintf ppf "%,@[<hv 2>h : key -> value =";
  pprint_h pp_key (printer_value pp_key)
    Og_precedence.Ck_record_field ppf t;
  Format.fprintf ppf "%,;@]";
  Format.fprintf ppf "%,@ ";
  Format.fprintf ppf "%,@[<hv 2>h_inv : value -> key =";
  pprint_h_inv pp_key (printer_value pp_key)
    Og_precedence.Ck_record_field ppf t;
  Format.fprintf ppf "%,;@]";
  Format.fprintf ppf "%,@]";
  if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
  then Format.fprintf ppf "%,@;<1 -2>}"
  else Format.fprintf ppf "%,@;<1 -2>}@]"
;;

(* Top printers. *)

let print_value pp_a = printer_value pp_a Og_precedence.Ck_toplevel;;

let print_t pp_size pp_key =
  printer_t pp_size pp_key Og_precedence.Ck_toplevel
;;
