(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.2+dev0 generated code. *)

let printer_bytes ctx ppf = function
  | u_b -> Upervasives_types_print.printer_bytes ctx ppf u_b
;;

let rec printer_software_name ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_module_name ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_function_name ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_tag ctx ppf = function | s -> Og_printer.printer_String ctx ppf s

and printer_prefix ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_message ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s

and printer_gen_message ctx ppf = function
  | f_u_m ->
    Og_printer.printer_Fun Og_printer.printer_Unit printer_message
      ctx
      ppf
      f_u_m

and printer_qualify ctx ppf = function
  | f_f_f ->
    Og_printer.printer_Fun printer_function_name printer_function_name
      ctx
      ppf
      f_f_f
;;

let printer_length ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_index ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_offset ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_select _ctx ppf = function
  | Upervasives.Keep ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Keep"
  | Upervasives.Forget ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Forget"
;;

let printer_gen_message_option ctx ppf = function
  | f_u_m_o ->
    Og_printer.printer_Fun
      Og_printer.printer_Unit
      (Og_printer.printer_Option printer_message)
      ctx
      ppf
      f_u_m_o

and printer_qualify_opt ctx ppf = function
  | f_f_o_f_o ->
    Og_printer.printer_Fun
      (Og_printer.printer_Option printer_function_name)
      (Og_printer.printer_Option printer_function_name)
      ctx
      ppf
      f_f_o_f_o
;;

(* Defining top printers. *)

let print_bytes ppf = printer_bytes Og_precedence.Ck_toplevel ppf
;;

let print_software_name ppf =
  printer_software_name Og_precedence.Ck_toplevel ppf

and print_module_name ppf = printer_module_name Og_precedence.Ck_toplevel ppf

and print_function_name ppf =
  printer_function_name Og_precedence.Ck_toplevel ppf

and print_tag ppf = printer_tag Og_precedence.Ck_toplevel ppf

and print_prefix ppf = printer_prefix Og_precedence.Ck_toplevel ppf

and print_message ppf = printer_message Og_precedence.Ck_toplevel ppf

and print_gen_message ppf = printer_gen_message Og_precedence.Ck_toplevel ppf

and print_qualify ppf = printer_qualify Og_precedence.Ck_toplevel ppf
;;

let print_length ppf = printer_length Og_precedence.Ck_toplevel ppf
;;

let print_index ppf = printer_index Og_precedence.Ck_toplevel ppf
;;

let print_offset ppf = printer_offset Og_precedence.Ck_toplevel ppf
;;

let print_select ppf = printer_select Og_precedence.Ck_toplevel ppf
;;

let print_gen_message_option ppf =
  printer_gen_message_option Og_precedence.Ck_toplevel ppf

and print_qualify_opt ppf = printer_qualify_opt Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.2+dev0 generated code. *)
