(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ublockmatrix.mli,v 1.16 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 Block matrix operations} *)

type 'a t = 'a Umatrix.t Umatrix.t
and 'a block_matrix = 'a t
(** The type of block matrices.
   A block matrix is a two-dimensional array of matrices. *)
;;

val size : 'a t -> Unatural.t * Unatural.t
(** Return the size of the given block matrix.
    The size of a block matrix is a pair of integers that gives the number
    of blocks in both directions. *)
;;

val length : 'a t -> Unatural.t * Unatural.t
(** [length] is an alias for {!size}. *)
;;

(*
val sizes : 'a t -> Unatural.t array * Unatural.t array
(** Return the sizes (pair of arrays of numbers of elements in both directions)
   of the given block matrix. *)
;;

val lengths : 'a t -> Unatural.t array * Unatural.t array
(** [lengths] is an alias for {!sizes}. *)
;;

val to_matrix : 'a t -> 'a Umatrix.t
(** *)
;;

val of_matrix :
  'a Umatrix.t -> Unatural.t array -> Unatural.t array -> 'a t
(** *)
;;
*)

val resize : Unatural.t -> Unatural.t -> 'a t -> 'a t
(** [resize ni nj m] returns a fresh matrix of size [(ni, nj)].
   Each element of the given matrix [m] becomes a submatrix of size

   Raise [Invalid_arg] if the matrix [m] has a zero dimension and not... . *)
;;

val zoom : Unatural.t -> Unatural.t -> 'a t -> 'a t
(** [zoom ni nj m] returns a fresh matrix whose dimensions are
   those of [m] multiplied by [ni] and [nj]. Each element of the given matrix
   [m] becomes a submatrix of size [(ni, nj)]. *)
;;
