(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ugraph.mli,v 1.7 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 Graphs of functions} *)

type 'a index = 'a Ugraph_types.index
and 'a uindex = 'a Ugraph_types.uindex
;;

type 'a domain = 'a Ugraph_types.domain
and 'a sub_domain = 'a Unonempty_set.nonempty_subset
;;

type 'b images = 'b Ugraph_types.images =
  | Constant of 'b
  | Images of 'b array
;;

type ('a, 'b) t = ('a, 'b) Ugraph_types.t = private
  {
   domain : 'a domain;
   images : 'b images;
  }
and ('a, 'b) graph = ('a, 'b) t
;;

val same : ('a, 'b) t -> ('a, 'b) t -> bool;;

(** {6 Construction functions} *)

val constant : 'a domain -> 'b -> ('a, 'b) t;;
val from_array : 'a domain -> 'b array -> ('a, 'b) t;;

val from_fun : 'a domain -> ('a index -> 'a uindex -> 'b) -> ('a, 'b) t;;

val of_matrix : 'a Umatrix.t -> (int * int, 'a) t;;

(** {6 Functions on graphs} *)

val image : ('a, 'b) t -> 'a index -> 'b;;

val restriction : ('a, 'b) t -> 'a sub_domain -> ('a, 'b) t;;

(** {6 Iterators} *)

val map_images : ('b -> 'c) -> ('a, 'b) t -> ('a, 'c) t;;
