(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ugraph_types.mli,v 1.2 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 The private type of graphs of functions} *)

type 'a index = 'a Unonempty_set.index
and 'a uindex = 'a Unonempty_set.uindex
;;

type 'a domain = 'a Unonempty_set.t;;

type 'b images =
  | Constant of 'b
  | Images of 'b array
;;

type ('a, 'b) t = private
  {
   domain : 'a domain;
   images : 'b images;
  }
and ('a, 'b) graph = ('a, 'b) t
;;

val constant : 'a domain -> 'b -> ('a, 'b) t;;
val from_array : 'a domain -> 'b array -> ('a, 'b) t;;
val from_fun : 'a domain -> ('a index -> 'a uindex -> 'b) -> ('a, 'b) t;;
val of_matrix : 'a Umatrix.t -> (int * int, 'a) t;;
val map_images : ('b -> 'c) -> ('a, 'b) t -> ('a, 'c) t;;
