(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uoption_key_types.ml,v 1.3 2017/05/31 15:24:06 fclement Exp $ *)

(* The private type of option keys. *)

module Nat = Unatural;;
module S = Ustring;;

let _0_ = Nat._0_
and _1_ = Nat._1_
;;

let qual = Upervasives.qualify "Uoption_key";;
let failwith = Upervasives.failwith_qual qual;;

type t = string
and option_key = t
;;

let is_dash c = c = '-';;

let fail = failwith "of_string";;
let fail_in s msg = fail (Printf.sprintf "%s in %S" msg s);;

let to_t s =
  let len = S.length s in
  if len = _0_ then fail "empty option key" else
  (* Now, we have len > 0.
     Therefore, the following S.unsafe_get will be safe. *)
  if not (is_dash (S.unsafe_get s _0_)) then fail_in s "no leading dash" else
  if len = _1_ then fail "single character option key" else
  s
;;

external of_t : t -> string = "%identity";;
