(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uversion.ml,v 1.10 2017/05/31 15:24:06 fclement Exp $ *)

(* Handling the user part of the id card of a Pph project. *)

module Co = Ucommand_option;;

(* Command options. *)

let version_rco k msg doc =
  {
   Co.key = k;
   spec = Arg.Unit (fun () -> Printf.eprintf "%s" msg; exit 0);
   doc = doc;
  }
;;

let gen_add
    (app_add_all, lib_add_all) purpose
    { Uoption_key.long = lk; short = sk_st; } msg doc =
  let rco_with_sko = [ (sk_st, version_rco lk msg doc); ] in
  match purpose with
  | Co.Application -> app_add_all rco_with_sko
  | Co.Library -> lib_add_all rco_with_sko
;;

let gen_add_long_version_option =
  gen_add
    (Co.long_version_info_app_add_all_rco_with_sk,
     Co.long_version_info_lib_add_all_rco_with_sk)
and gen_add_version_option =
  gen_add
    (Co.version_info_app_add_all_rco_with_sk,
     Co.version_info_lib_add_all_rco_with_sk)
and gen_add_short_version_option =
  gen_add
    (Co.short_version_info_app_add_all_rco_with_sk,
     Co.short_version_info_lib_add_all_rco_with_sk)
;;

let add_app_long_version_option =
  gen_add_long_version_option Co.Application
and add_lib_long_version_option =
  gen_add_long_version_option Co.Library
and add_app_version_option =
  gen_add_version_option Co.Application
and add_lib_version_option =
  gen_add_version_option Co.Library
and add_app_short_version_option =
  gen_add_short_version_option Co.Application
and add_lib_short_version_option =
  gen_add_short_version_option Co.Library
;;

let add_application_version_option = add_app_version_option
and add_library_version_option = add_lib_version_option
;;
